/*
 * @(#)Light3D.java    Pzt 10 May 2010 14:32:03 EEST
 *
 * Light 3d for 3d world
 */
/*
*** This program wrote under GNU/General Public License ***

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.
*/
package net.dilmerkezi.light;

import java.awt.Color;

//custom
import net.dilmerkezi.point.*;


/**
 * @author Murat iNAN, dilmerkezi.net/eng.html
 */
//final
public class Light3D
     extends Object
  implements Light
{

    /** Light3D origin*/
    private Point3D origin;
    
    /** Light color*/
    private Color lightColor;
    
    /** First constructor*/
    public Light3D ()
    {
        this.origin = new Vector3D ();
        lightColor = new Color (0xffffff);
    }
    
    /**
     * Second constructor
     *
     * @param p New origin point
     */
    public Light3D (Point3D p)
    {
        this.origin = p;
    }
    
    /**
     * Third constructor
     *
     * @param p New origin point
     * @param c New light color
     */
    public Light3D (Point3D p, Color c)
    {
        this.origin = p;
        this.lightColor = c;
    }
    
    /**
     * @return Light3D origin 3D point
     */
    public Point3D getLightOriginPoint3D ()
    {
        return origin;
    }

    /** @return Light3D color*/
    public Color getLightColor ()
    {
        return lightColor;
    }
    
    /**
     * @param point Light3D point
     * @param normal Normalized point
     * @param camera Camera point
     *
     * @return Speculaded value
     */
    public double specular (Point3D point, Point3D normal, Point3D camera)
    {
        Point3D normalnormal = normal.normalize ();
        Point3D sub = origin.subtract (point);
        Point3D subnormal = sub.normalize ();
        Point3D eye = camera.subtract (point);
        Point3D eyenormal = eye.normalize ();
        Point3D reflect = normalnormal.reflect (subnormal);
        double intensity = eyenormal.dot (reflect);
        if (intensity < 0.001) return 0.0;
        double pow = Math.pow (intensity, 100d);
        return pow;
    }

    /**
     * @param point Light3D point
     * @param normal Normalized point
     *
     * @return Diffused value
     */
    public double diffuse (Point3D point, Point3D normal)
    {
        Point3D sub = origin.subtract (point);
        Point3D subnormal = sub.normalize ();
        Point3D normalnormal = normal.normalize ();
        double sin = normalnormal.dot (subnormal);
        if (sin < 0.0) return 0.0;
        return sin;
    }
    
    /**
     * @param lg Comparation light
     * @return If equals true, otherwise false;
     */
    public boolean equals (Light3D lg)
    {
        return (origin.equals (lg.getLightOriginPoint3D ()) &&
                lightColor.toString ().equals (lg.getLightColor ().toString ()));
    }

    /** @return hashcode*/
    public int hashCode ()
    {
        String s = toString ();
        int result = 0;
        char [] ch = s.toCharArray ();
        int chlen = ch.length;
        int ff = 0xff;
        for (int i=0; i<chlen; i++)
        {
            result += ( ((int)(ch [i]))*ff );
        }
        return result;
    }
    
        /** @return Description*/
    public String toString ()
    {
        StringBuffer sb = new StringBuffer ();
        sb.append (origin.toString ());
        sb.append ("\n---------\n");
        sb.append (lightColor.toString ());
        sb.append ("\n");
        return sb.toString ();
    }
    
}
/****************/
